<?php

namespace Drupal\bookable_calendar\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\bookable_calendar\Entity\BookableCalendarOpeningInstance;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for Bookable Calendar routes.
 */
class BookableCalendarApiController extends ControllerBase {
  protected $entity_type_manager;
  protected $instance_storage;
  protected $contact_storage;

  function __construct()
  {
    $this->entity_type_manager = \Drupal::service('entity_type.manager');
    $this->instance_storage = $this->entity_type_manager->getStorage('bookable_calendar_opening_inst');
    $this->contact_storage = $this->entity_type_manager->getStorage('booking_contact');
  }

  public function book(BookableCalendarOpeningInstance $opening_instance, Request $req) {
    // allow for both POST body and form-data
    $post_body = $req->getContent();
    $post_form_data = $req->request->all();
    if ($post_body || $post_form_data) {
      if ($post_body) {
        $body = (array) json_decode($post_body);
        $contact = (array) $body['contact_info'];
      }
      else {
        $contact = $post_form_data;
      }

      $contact['booking_instance'] = [
        'target_id' => $opening_instance->id()
      ];

      if (isset($contact['email']) && isset($contact['party_size'])) {
        $new_contact = $this->contact_storage->create($contact);
        $violations = $new_contact->validate();

        if ($violations->count() > 0) {
          return new JsonResponse([
            'status' => 'failed',
            'message' => 'There was some validation errors'
          ]);
        }
        else {
          $new_contact->save();
          $sucess_message = $opening_instance->getSuccessMessage();
          return new JsonResponse([
            'status' => 'success',
            'message' => $sucess_message
          ]);
        }
      }
      else {
        return new JsonResponse([
          'status' => 'failed',
          'message' => 'not all required parameters were provided'
        ]);
      }
    }
    else {
      return new JsonResponse([
        'status' => 'failed',
        'message' => 'No request body sent'
      ]);
    }
  }
}
